﻿/*
Copyright (c) 2014 Stephen Stair (sgstair@akkit.org)

Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
*/

// 
// the winusbdotnet repo isn't the correct place for this code long term
// Code is here for now for convenience in testing and iteration while being developed.
//
#define AVERAGE

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;
using System.Windows.Forms;

using winusbdotnet.UsbDevices;

namespace TestSeek
{
    public partial class Form1 : Form
    {

        SeekThermal thermal;
        Thread thermalThread;
        int frameCount;
        bool stopThread;

        ThermalFrame lastFrame, lastCalibrationFrame;
        CalibratedThermalFrame lastUsableFrame;
        CalibratedThermalFrame lastRenderedFrame;

        byte[] FrameData;
        Queue<Bitmap> bmpQueue;

        public Form1()
        {
            InitializeComponent();

            DoubleBuffered = true;
            bmpQueue = new Queue<Bitmap>();

            var device = SeekThermal.Enumerate().FirstOrDefault();
            if(device == null)
            {
                MessageBox.Show("No Seek Thermal devices found.");
                return;
            }
            thermal = new SeekThermal(device);

            thermalThread = new Thread(ThermalThreadProc);
            thermalThread.IsBackground = true;
            thermalThread.Start();
        }

        void ThermalThreadProc()
        {
            while (!stopThread && thermal != null)
            {
                bool progress = false;
                lastFrame = thermal.GetFrameBlocking();

                if(lastFrame.IsCalibrationFrame)
                {
                    lastCalibrationFrame = lastFrame;
                }
                else
                {
                    if(lastCalibrationFrame != null && lastFrame.IsUsableFrame)
                    {
                        lastUsableFrame = lastFrame.ProcessFrame(lastCalibrationFrame);
                        progress = true;
                    }
                }


                //System.Diagnostics.Debug.Print("Start of data: " + string.Join(" ", lastFrame.RawData.Take(64).Select(b => b.ToString("x2")).ToArray()));
                //System.Diagnostics.Debug.Print("End of data: " + string.Join(" ", lastFrame.RawData.Reverse().Take(32).Reverse().Select(b => b.ToString("x2")).ToArray()));
                
                // Do stuff
                frameCount++;
                if(progress)
                {
                    Invalidate();
                }
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            stopThread = true;
            if (thermal != null)
            {
                thermalThread.Join(500);
                thermal.Deinit();
            }
        }

        public struct palette
        {
            public int r, g, b;
            public palette(int ri, int gi, int bi)
            {
                r = ri;
                g = gi;
                b = bi;
            }
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            CalibratedThermalFrame data = lastUsableFrame;
            if (data == null) return;
            int y;
            if(data != lastRenderedFrame)
            {
                lastRenderedFrame = data;
                // Process new frame
                Bitmap bmp = new Bitmap(data.Width, data.Height);
                int c = 0;
#if AVERAGE
                palette[] IronPalette = {
                    new palette(0, 0, 0),
                    new palette(0, 0, 7),
                    new palette(0, 0, 24),
                    new palette(0, 0, 38),
                    new palette(0, 0, 46),
                    new palette(0, 0, 52),
                    new palette(0, 0, 59),
                    new palette(0, 0, 66),
                    new palette(0, 0, 73),
                    new palette(0, 0, 80),
                    new palette(1, 0, 85),
                    new palette(2, 0, 89),
                    new palette(3, 0, 93),
                    new palette(4, 0, 97),
                    new palette(5, 0, 101),
                    new palette(6, 0, 104),
                    new palette(8, 0, 108),
                    new palette(10, 0, 112),
                    new palette(11, 0, 116),
                    new palette(13, 0, 118),
                    new palette(15, 0, 119),
                    new palette(17, 0, 121),
                    new palette(20, 0, 123),
                    new palette(24, 0, 126),
                    new palette(27, 0, 128),
                    new palette(30, 0, 130),
                    new palette(33, 0, 133),
                    new palette(36, 0, 135),
                    new palette(40, 0, 136),
                    new palette(43, 0, 138),
                    new palette(47, 0, 139),
                    new palette(50, 0, 141),
                    new palette(53, 0, 142),
                    new palette(56, 0, 144),
                    new palette(59, 0, 145),
                    new palette(62, 0, 146),
                    new palette(64, 0, 148),
                    new palette(66, 0, 149),
                    new palette(69, 0, 150),
                    new palette(72, 0, 150),
                    new palette(75, 0, 150),
                    new palette(78, 0, 151),
                    new palette(81, 0, 151),
                    new palette(83, 0, 152),
                    new palette(87, 0, 152),
                    new palette(90, 0, 153),
                    new palette(93, 0, 154),
                    new palette(96, 0, 155),
                    new palette(99, 0, 155),
                    new palette(102, 0, 155),
                    new palette(105, 0, 155),
                    new palette(108, 0, 156),
                    new palette(111, 0, 156),
                    new palette(113, 0, 157),
                    new palette(115, 0, 157),
                    new palette(119, 0, 157),
                    new palette(122, 0, 157),
                    new palette(125, 0, 157),
                    new palette(128, 0, 157),
                    new palette(130, 0, 157),
                    new palette(133, 0, 157),
                    new palette(136, 0, 157),
                    new palette(138, 0, 157),
                    new palette(141, 0, 157),
                    new palette(144, 0, 156),
                    new palette(147, 0, 156),
                    new palette(150, 0, 155),
                    new palette(153, 0, 155),
                    new palette(155, 0, 155),
                    new palette(157, 0, 155),
                    new palette(160, 0, 155),
                    new palette(162, 0, 155),
                    new palette(164, 0, 155),
                    new palette(167, 0, 154),
                    new palette(169, 0, 154),
                    new palette(171, 0, 153),
                    new palette(172, 0, 153),
                    new palette(174, 1, 152),
                    new palette(176, 1, 152),
                    new palette(177, 1, 151),
                    new palette(179, 1, 151),
                    new palette(181, 2, 150),
                    new palette(183, 2, 149),
                    new palette(184, 3, 149),
                    new palette(185, 4, 149),
                    new palette(187, 5, 148),
                    new palette(188, 5, 147),
                    new palette(190, 5, 146),
                    new palette(191, 6, 146),
                    new palette(192, 7, 145),
                    new palette(193, 8, 144),
                    new palette(194, 9, 143),
                    new palette(195, 11, 142),
                    new palette(196, 12, 141),
                    new palette(197, 13, 140),
                    new palette(199, 14, 138),
                    new palette(200, 16, 136),
                    new palette(201, 18, 134),
                    new palette(202, 19, 133),
                    new palette(203, 20, 131),
                    new palette(205, 22, 129),
                    new palette(206, 23, 127),
                    new palette(207, 25, 124),
                    new palette(208, 26, 121),
                    new palette(209, 27, 119),
                    new palette(210, 29, 116),
                    new palette(211, 31, 114),
                    new palette(212, 33, 112),
                    new palette(213, 34, 108),
                    new palette(214, 36, 104),
                    new palette(215, 38, 101),
                    new palette(216, 40, 98),
                    new palette(217, 42, 95),
                    new palette(218, 45, 92),
                    new palette(219, 47, 87),
                    new palette(220, 48, 82),
                    new palette(221, 49, 77),
                    new palette(222, 51, 71),
                    new palette(223, 53, 65),
                    new palette(224, 55, 60),
                    new palette(224, 56, 54),
                    new palette(225, 58, 49),
                    new palette(226, 60, 43),
                    new palette(227, 62, 37),
                    new palette(228, 63, 32),
                    new palette(228, 65, 28),
                    new palette(229, 67, 26),
                    new palette(230, 69, 24),
                    new palette(230, 71, 21),
                    new palette(231, 73, 19),
                    new palette(232, 75, 17),
                    new palette(232, 76, 15),
                    new palette(233, 77, 13),
                    new palette(234, 78, 12),
                    new palette(235, 80, 11),
                    new palette(235, 82, 10),
                    new palette(235, 84, 9),
                    new palette(236, 86, 8),
                    new palette(236, 88, 8),
                    new palette(237, 90, 7),
                    new palette(237, 91, 6),
                    new palette(238, 92, 5),
                    new palette(238, 94, 5),
                    new palette(239, 95, 4),
                    new palette(239, 97, 4),
                    new palette(240, 99, 3),
                    new palette(240, 101, 3),
                    new palette(241, 102, 3),
                    new palette(241, 103, 3),
                    new palette(241, 105, 2),
                    new palette(241, 106, 2),
                    new palette(241, 108, 1),
                    new palette(242, 109, 1),
                    new palette(242, 111, 1),
                    new palette(243, 113, 1),
                    new palette(243, 114, 1),
                    new palette(244, 116, 0),
                    new palette(244, 117, 0),
                    new palette(244, 119, 0),
                    new palette(244, 121, 0),
                    new palette(245, 124, 0),
                    new palette(245, 126, 0),
                    new palette(246, 128, 0),
                    new palette(246, 130, 0),
                    new palette(247, 131, 0),
                    new palette(247, 133, 0),
                    new palette(248, 135, 0),
                    new palette(248, 136, 0),
                    new palette(248, 137, 0),
                    new palette(248, 139, 0),
                    new palette(248, 140, 0),
                    new palette(249, 142, 0),
                    new palette(249, 143, 0),
                    new palette(249, 144, 0),
                    new palette(249, 146, 0),
                    new palette(250, 148, 0),
                    new palette(250, 150, 0),
                    new palette(251, 152, 0),
                    new palette(251, 155, 0),
                    new palette(252, 157, 0),
                    new palette(252, 159, 0),
                    new palette(253, 161, 0),
                    new palette(253, 163, 0),
                    new palette(253, 166, 0),
                    new palette(253, 168, 0),
                    new palette(253, 170, 0),
                    new palette(253, 172, 0),
                    new palette(253, 174, 0),
                    new palette(254, 175, 0),
                    new palette(254, 177, 0),
                    new palette(254, 178, 0),
                    new palette(254, 180, 0),
                    new palette(254, 183, 0),
                    new palette(254, 185, 0),
                    new palette(254, 186, 0),
                    new palette(254, 188, 0),
                    new palette(254, 189, 0),
                    new palette(254, 191, 0),
                    new palette(254, 193, 0),
                    new palette(254, 195, 0),
                    new palette(254, 197, 0),
                    new palette(254, 199, 0),
                    new palette(254, 200, 0),
                    new palette(254, 202, 1),
                    new palette(254, 203, 1),
                    new palette(254, 204, 2),
                    new palette(254, 206, 3),
                    new palette(254, 207, 4),
                    new palette(254, 209, 6),
                    new palette(254, 211, 8),
                    new palette(254, 213, 10),
                    new palette(254, 215, 11),
                    new palette(254, 216, 12),
                    new palette(255, 218, 14),
                    new palette(255, 219, 16),
                    new palette(255, 220, 19),
                    new palette(255, 221, 23),
                    new palette(255, 222, 27),
                    new palette(255, 224, 31),
                    new palette(255, 225, 35),
                    new palette(255, 227, 38),
                    new palette(255, 228, 42),
                    new palette(255, 229, 48),
                    new palette(255, 230, 53),
                    new palette(255, 231, 60),
                    new palette(255, 233, 65),
                    new palette(255, 234, 71),
                    new palette(255, 235, 77),
                    new palette(255, 237, 83),
                    new palette(255, 238, 89),
                    new palette(255, 239, 96),
                    new palette(255, 239, 102),
                    new palette(255, 240, 109),
                    new palette(255, 241, 115),
                    new palette(255, 241, 124),
                    new palette(255, 242, 132),
                    new palette(255, 243, 139),
                    new palette(255, 244, 146),
                    new palette(255, 244, 153),
                    new palette(255, 245, 160),
                    new palette(255, 245, 168),
                    new palette(255, 246, 175),
                    new palette(255, 247, 181),
                    new palette(255, 248, 187),
                    new palette(255, 248, 193),
                    new palette(255, 249, 198),
                    new palette(255, 249, 204),
                    new palette(255, 250, 210),
                    new palette(255, 251, 216),
                    new palette(255, 252, 222),
                    new palette(255, 253, 227),
                    new palette(255, 253, 232),
                    new palette(255, 254, 237),
                    new palette(255, 254, 243),
                    new palette(255, 255, 247),
                    new palette(255, 255, 249)
                };

                int[] deadpixelarray = { 10, 6, 2, 13, 9, 5, 1, 12, 8, 4, 0, 11, 7, 3, 14 };
                int deadpixel;
                ulong average;
                ulong numpoints;
                int v;
                for (y = 0; y < 156; y++)
                {
                    deadpixel = deadpixelarray[y % 15];
                    for (int x = 0; x < 208; x++)
                    {
                        if (x == deadpixel)
                        {
                            average = 0UL;
                            numpoints = 0;
                            if (x > 0)
                            {
                                average += data.PixelData[c - 1];
                                numpoints++;
                            }
                            if (x < 208)
                            {
                                average += data.PixelData[c + 1];
                                numpoints++;
                            }
                            if (y > 0)
                            {
                                average += data.PixelData[c - 208];
                                numpoints++;
                            }
                            if (y < 155)
                            {
                                average += data.PixelData[c + 208];
                                numpoints++;
                            }
                            if (numpoints > 0)
                                average = average / numpoints;
                            else
                                average = data.MinValue;
                            v = (int)average;
                            deadpixel += 15;
                        }
                        else
                        {
                            v = data.PixelData[c];
                        }
                        v = (v - data.MinValue) * 255 / (data.MaxValue - data.MinValue);

                        if (v < 0) v = 0;
                        if (v > 255) v = 255;
                        bmp.SetPixel(x, y, Color.FromArgb(IronPalette[v].r, IronPalette[v].g, IronPalette[v].b));
                        c++;
                    }
                }
#else
                for (y = 0; y < 156; y++)
                {
                    for (int x = 0; x < 208; x++)
                    {
                        int v = data.PixelData[c++];

                        v = (v - data.MinValue) * 255 / (data.MaxValue - data.MinValue);
                        if (v < 0) v = 0;
                        if (v > 255) v = 255;

                        bmp.SetPixel(x, y, Color.FromArgb(v, v, v));
                    }
                }
#endif
                bmpQueue.Enqueue(bmp);
                if (bmpQueue.Count > 3) bmpQueue.Dequeue(); 
            }

            y = 10;
            foreach(Bitmap b in bmpQueue.Reverse())
            {
                e.Graphics.DrawImage(b, 10, y);
                y += b.Height + 10;
            }
        }
    }
}
